#include <memory>
#include "lib.h"

class Callbacks final : public Dynarmic::A32::UserCallbacks {
public:
    std::uint8_t MemoryRead8(std::uint32_t vaddr) override {
        return CbMemoryRead8(static_cast<std::size_t>(vaddr));
    }

    std::uint16_t MemoryRead16(std::uint32_t vaddr) override {
        return CbMemoryRead16(static_cast<std::size_t>(vaddr));
    }

    std::uint32_t MemoryRead32(std::uint32_t vaddr) override {
        return CbMemoryRead32(static_cast<std::size_t>(vaddr));
    }

    std::uint64_t MemoryRead64(std::uint32_t vaddr) override {
        return CbMemoryRead64(static_cast<std::size_t>(vaddr));
    }

    void MemoryWrite8(std::uint32_t vaddr, std::uint8_t value) override {
        CbMemoryWrite8(static_cast<std::size_t>(vaddr), value);
    }

    void MemoryWrite16(std::uint32_t vaddr, std::uint16_t value) override {
        CbMemoryWrite16(static_cast<std::size_t>(vaddr), value);
    }

    void MemoryWrite32(std::uint32_t vaddr, std::uint32_t value) override {
        CbMemoryWrite32(static_cast<std::size_t>(vaddr), value);
    }

    void MemoryWrite64(std::uint32_t vaddr, std::uint64_t value) override {
        CbMemoryWrite64(static_cast<std::size_t>(vaddr), value);
    }

    void InterpreterFallback(std::uint32_t pc, std::size_t num_instructions) override {
        CbInterpreterFallback(pc, num_instructions);
    }

    void CallSVC(std::uint32_t swi) override {
        CbCallSVC(swi);
    }

    void ExceptionRaised(std::uint32_t pc, Dynarmic::A32::Exception exception) override {
        CbExceptionRaised(pc, exception);
    }

    void AddTicks(std::uint64_t ticks) override {
        CbAddTicks(ticks);
    }

    std::uint64_t GetTicksRemaining() override {
        return CbGetTicksRemaining();
    }
};

std::unique_ptr<Dynarmic::A32::Jit> cpu;

void Initialize(bool define_unpredictable_behaviour, bool enable_fast_dispatch) {
    static Callbacks callbacks;

    Dynarmic::A32::UserConfig config;
    config.callbacks = &callbacks;
    config.define_unpredictable_behaviour = define_unpredictable_behaviour;
    config.enable_fast_dispatch = enable_fast_dispatch;

    cpu = std::make_unique<Dynarmic::A32::Jit>(config);
}

void Shutdown() {
    cpu.reset();
}

void Run() {
    cpu->Run();
}

void SetRegister(std::size_t n, std::uint32_t value) {
    cpu->Regs()[n] = value;
}

std::uint32_t GetRegister(std::size_t n) {
    return cpu->Regs()[n];
}

void SetCpsr(std::uint32_t value) {
    cpu->SetCpsr(value);
}

std::uint32_t GetCpsr() {
    return cpu->Cpsr();
}
