#pragma once

#include <cstdint>
#include <dynarmic/A32/a32.h>
#include <dynarmic/A32/config.h>

#define Callback(n, rt, ...) \
    using CbType ##n = rt(*)(__VA_ARGS__); \
    CbType ##n Cb##n; \
    void Set##n(CbType ##n cb) { \
        Cb##n = cb; \
    }

extern "C" {
void Initialize(bool define_unpredictable_behaviour, bool enable_fast_dispatch);
void Shutdown();
void Run();

void SetRegister(std::size_t n, std::uint32_t value);
std::uint32_t GetRegister(std::size_t n);

void SetCpsr(std::uint32_t value);
std::uint32_t GetCpsr();

Callback(MemoryRead8, std::uint8_t, std::size_t)
Callback(MemoryRead16, std::uint16_t, std::size_t)
Callback(MemoryRead32, std::uint32_t, std::size_t)
Callback(MemoryRead64, std::uint64_t, std::size_t)
Callback(MemoryWrite8, void, std::size_t, std::uint8_t)
Callback(MemoryWrite16, void, std::size_t, std::uint16_t)
Callback(MemoryWrite32, void, std::size_t, std::uint32_t)
Callback(MemoryWrite64, void, std::size_t, std::uint64_t)
Callback(InterpreterFallback, void, std::uint32_t, std::size_t)
Callback(CallSVC, void, std::uint32_t)
Callback(ExceptionRaised, void, std::uint32_t, Dynarmic::A32::Exception)
Callback(AddTicks, void, std::uint64_t)
Callback(GetTicksRemaining, std::uint64_t)
}
